@@ -9,7 +9,7 @@ use bcder::encode::{PrimitiveContent, Values};
 use super::oid;
 use super::cert::{Cert, ResourceCert};
 use super::crypto::{Signer, SigningError};
-use super::resources::{Addr, AddressFamily, AsId, IpResources, Prefix};
+use super::resources::{Addr, AddressFamily, Asn, IpResources, Prefix};
 use super::sigobj::{SignedObject, SignedObjectBuilder};
 use super::x509::ValidationError;
 
@@ -99,13 +99,13 @@ impl<'de> serde::Deserialize<'de> for Roa {
 
 #[derive(Clone, Debug)]
 pub struct RouteOriginAttestation {
-    as_id: AsId,
+    as_id: Asn,
     v4_addrs: RoaIpAddresses,
     v6_addrs: RoaIpAddresses,
 }
 
 impl RouteOriginAttestation {
-    pub fn as_id(&self) -> AsId {
+    pub fn as_id(&self) -> Asn {
         self.as_id
     }
 
@@ -135,7 +135,7 @@ impl RouteOriginAttestation {
         cons.take_sequence(|cons| {
             // version [0] EXPLICIT INTEGER DEFAULT 0
             cons.take_opt_constructed_if(Tag::CTX_0, |c| c.skip_u8_if(0))?;
-            let as_id = AsId::take_from(cons)?;
+            let as_id = Asn::take_from(cons)?;
             let mut v4 = None;
             let mut v6 = None;
             cons.take_sequence(|cons| {
@@ -145,13 +145,17 @@ impl RouteOriginAttestation {
                             if v4.is_some() {
                                 xerr!(return Err(decode::Malformed.into()));
                             }
-                            v4 = Some(RoaIpAddresses::take_from(cons)?);
+                            v4 = Some(RoaIpAddresses::take_from(
+                                cons, AddressFamily::Ipv4
+                            )?);
                         }
                         AddressFamily::Ipv6 => {
                             if v6.is_some() {
                                 xerr!(return Err(decode::Malformed.into()));
                             }
-                            v6 = Some(RoaIpAddresses::take_from(cons)?);
+                            v6 = Some(RoaIpAddresses::take_from(
+                                cons, AddressFamily::Ipv6
+                            )?);
                         }
                     }
                     Ok(())
@@ -222,11 +226,12 @@ pub struct RoaIpAddresses(Captured);
 
 impl RoaIpAddresses {
     fn take_from<S: decode::Source>(
-        cons: &mut decode::Constructed<S>
+        cons: &mut decode::Constructed<S>,
+        family: AddressFamily,
     ) -> Result<Self, S::Err> {
         cons.take_sequence(|cons| {
             cons.capture(|cons| {
-                while let Some(()) = RoaIpAddress::skip_opt_in(cons)? { }
+                while RoaIpAddress::skip_opt_in(cons, family)?.is_some() { }
                 Ok(())
             })
         }).map(RoaIpAddresses)
@@ -272,7 +277,7 @@ impl<'a> Iterator for RoaIpAddressIter<'a> {
         }
         else {
             Mode::Der.decode(&mut self.0, |cons| {
-                RoaIpAddress::take_opt_from(cons)
+                RoaIpAddress::take_opt_from_unchecked(cons)
             }).unwrap()
         }
     }
@@ -319,7 +324,7 @@ impl RoaIpAddress {
     // The address is the same as in section 2.1.1 of RFC 3779, that is, it
     // is a bit string with all the bits of the prefix.
 
-    fn take_opt_from<S: decode::Source>(
+    fn take_opt_from_unchecked<S: decode::Source>(
         cons: &mut decode::Constructed<S>
     ) -> Result<Option<Self>, S::Err> {
         cons.take_opt_sequence(|cons| {
@@ -330,10 +335,34 @@ impl RoaIpAddress {
         })
     }
 
+    /// Skips one address in a source.
+    ///
+    /// In order to check that the address is correctly formatted, this
+    /// function needs to know the address family of the address.
     fn skip_opt_in<S: decode::Source>(
-        cons: &mut decode::Constructed<S>
+        cons: &mut decode::Constructed<S>,
+        family: AddressFamily,
     ) -> Result<Option<()>, S::Err> {
-        Self::take_opt_from(cons).map(|res| res.map(|_| ()))
+        let addr = match Self::take_opt_from_unchecked(cons)? {
+            Some(addr) => addr,
+            None => return Ok(None)
+        };
+
+        // Check that the prefix length fits the address family.
+        if addr.prefix.addr_len() > family.max_addr_len() {
+            return Err(decode::Malformed.into())
+        }
+
+        // Check that a max length fits both family and prefix length.
+        if let Some(max_length) = addr.max_length {
+            if max_length > family.max_addr_len() 
+                || max_length < addr.prefix.addr_len()
+            {
+                return Err(decode::Malformed.into())
+            }
+        }
+
+        Ok(Some(()))
     }
 
     fn encode(&self) -> impl encode::Values {
@@ -390,13 +419,13 @@ impl FriendlyRoaIpAddress {
 //------------ RoaBuilder ----------------------------------------------------
 
 pub struct RoaBuilder {
-    as_id: AsId,
+    as_id: Asn,
     v4: RoaIpAddressesBuilder,
     v6: RoaIpAddressesBuilder,
 }
 
 impl RoaBuilder {
-    pub fn new(as_id: AsId) -> Self {
+    pub fn new(as_id: Asn) -> Self {
         Self::with_addresses(
             as_id,
             RoaIpAddressesBuilder::new(),
@@ -405,18 +434,18 @@ impl RoaBuilder {
     }
 
     pub fn with_addresses(
-        as_id: AsId,
+        as_id: Asn,
         v4: RoaIpAddressesBuilder,
         v6: RoaIpAddressesBuilder
     ) -> Self {
         Self { as_id, v4, v6 }
     }
 
-    pub fn as_id(&self) -> AsId {
+    pub fn as_id(&self) -> Asn {
         self.as_id
     }
 
-    pub fn set_as_id(&mut self, as_id: AsId) {
+    pub fn set_as_id(&mut self, as_id: Asn) {
         self.as_id = as_id
     }
 
@@ -590,6 +619,30 @@ mod test {
             ).is_ok()
         )
     }
+
+    #[test]
+    fn decode_illegal_roas() {
+        assert!(
+            Roa::decode(
+                include_bytes!(
+                    "../../test-data/prefix-len-overflow.roa"
+                ).as_ref(),
+                false
+            ).is_err()
+        );
+        assert!(
+            Roa::decode(
+                include_bytes!("../../test-data/maxlen-overflow.roa").as_ref(),
+                false
+            ).is_err()
+        );
+        assert!(
+            Roa::decode(
+                include_bytes!("../../test-data/maxlen-underflow.roa").as_ref(),
+                false
+            ).is_err()
+        );
+    }
 }
 
 #[cfg(all(test, feature="softkeys"))]
@@ -600,13 +653,13 @@ mod signer_test {
     use crate::repository::cert::{KeyUsage, Overclaim, TbsCert};
     use crate::repository::crypto::{PublicKeyFormat, Signer};
     use crate::repository::crypto::softsigner::OpenSslSigner;
-    use crate::repository::resources::{AsId, Prefix};
+    use crate::repository::resources::{Asn, Prefix};
     use crate::repository::tal::TalInfo;
     use crate::repository::x509::Validity;
     use super::*;
 
     fn make_roa() -> Roa {
-        let mut signer = OpenSslSigner::new();
+        let signer = OpenSslSigner::new();
         let key = signer.create_key(PublicKeyFormat::Rsa).unwrap();
         let pubkey = signer.get_key_info(&key).unwrap();
         let uri = uri::Rsync::from_str("rsync://example.com/m/p").unwrap();
@@ -621,7 +674,7 @@ mod signer_test {
         cert.set_rpki_manifest(Some(uri.clone()));
         cert.build_v4_resource_blocks(|b| b.push(Prefix::new(0, 0)));
         cert.build_v6_resource_blocks(|b| b.push(Prefix::new(0, 0)));
-        cert.build_as_resource_blocks(|b| b.push((AsId::MIN, AsId::MAX)));
+        cert.build_as_resource_blocks(|b| b.push((Asn::MIN, Asn::MAX)));
         let cert = cert.into_cert(&signer, &key).unwrap();
 
         let mut roa = RoaBuilder::new(64496.into());
@@ -630,7 +683,7 @@ mod signer_test {
         let roa = roa.finalize(
             SignedObjectBuilder::new(
                 12u64.into(), Validity::from_secs(86400), uri.clone(),
-                uri.clone(), uri.clone()
+                uri.clone(), uri
             ),
             &signer, &key
         ).unwrap();
